@@ -1,9 +1,8 @@
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
-use rusty_kernel_tools::*;
-use rusty_green_kernel::*;
-use ndarray;
-use num::complex::Complex;
+use num_cpus;
 use rand::Rng;
+use rusty_green_kernel::*;
+use ndarray_linalg::{c32, c64};
 
 fn benchmark_laplace_assemble_double_precision(c: &mut Criterion) {
     let nsources = 20000;
@@ -20,8 +19,7 @@ fn benchmark_laplace_assemble_double_precision(c: &mut Criterion) {
 
     c.bench_function("laplace assemble double precision", |b| {
         b.iter(|| {
-            make_laplace_evaluator(sources.view(), targets.view())
-                .assemble_in_place(black_box(result.view_mut()), ThreadingType::Parallel);
+            f64::assemble_kernel_in_place(sources.view(), targets.view(), black_box(result.view_mut()), KernelType::Laplace, num_cpus::get());
         })
     });
 }
@@ -41,8 +39,7 @@ fn benchmark_laplace_assemble_single_precision(c: &mut Criterion) {
 
     c.bench_function("laplace assemble single precision", |b| {
         b.iter(|| {
-            make_laplace_evaluator(sources.view(), targets.view())
-                .assemble_in_place(black_box(result.view_mut()), ThreadingType::Parallel);
+            f32::assemble_kernel_in_place(sources.view(), targets.view(), black_box(result.view_mut()), KernelType::Laplace, num_cpus::get());
         })
     });
 }
@@ -67,12 +64,8 @@ fn benchmark_laplace_evaluate_values_double_precision(c: &mut Criterion) {
 
     c.bench_function("laplace evaluate values double precision", |b| {
         b.iter(|| {
-            make_laplace_evaluator(sources.view(), targets.view()).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::Value,
-                ThreadingType::Parallel,
-            );
+            MyType::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Laplace, EvalMode::Value, num_cpus::get());
+
         })
     });
 }
@@ -97,12 +90,7 @@ fn benchmark_laplace_evalaute_values_single_precision(c: &mut Criterion) {
 
     c.bench_function("laplace evaluate values single precision", |b| {
         b.iter(|| {
-            make_laplace_evaluator(sources.view(), targets.view()).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::Value,
-                ThreadingType::Parallel,
-            );
+            MyType::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Laplace, EvalMode::Value, num_cpus::get());
         })
     });
 }
@@ -127,12 +115,7 @@ fn benchmark_laplace_evaluate_values_and_derivs_double_precision(c: &mut Criteri
 
     c.bench_function("laplace evaluate values and derivs double precision", |b| {
         b.iter(|| {
-            make_laplace_evaluator(sources.view(), targets.view()).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::ValueGrad,
-                ThreadingType::Parallel,
-            );
+            MyType::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Laplace, EvalMode::ValueGrad, num_cpus::get());
         })
     });
 }
@@ -157,12 +140,7 @@ fn benchmark_laplace_evaluate_values_and_derivs_single_precision(c: &mut Criteri
 
     c.bench_function("laplace evaluate values and derivs single precision", |b| {
         b.iter(|| {
-            make_laplace_evaluator(sources.view(), targets.view()).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::ValueGrad,
-                ThreadingType::Parallel,
-            );
+            MyType::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Laplace, EvalMode::ValueGrad, num_cpus::get());
         })
     });
 }
@@ -171,7 +149,7 @@ fn benchmark_helmholtz_assemble_single_precision(c: &mut Criterion) {
     let nsources = 20000;
     let ntargets = 20000;
 
-    let wavenumber = Complex::new(2.5, 0.0);
+    let wavenumber = c64::new(2.5, 0.0);
 
     type MyType = f32;
 
@@ -179,15 +157,14 @@ fn benchmark_helmholtz_assemble_single_precision(c: &mut Criterion) {
 
     let mut sources = ndarray::Array2::<MyType>::zeros((3, nsources));
     let mut targets = ndarray::Array2::<MyType>::zeros((3, ntargets));
-    let mut result = ndarray::Array2::<Complex<MyType>>::zeros((ntargets, nsources));
+    let mut result = ndarray::Array2::<c32>::zeros((ntargets, nsources));
 
     sources.map_inplace(|item| *item = rng.gen::<MyType>());
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
 
     c.bench_function("helmholtz assemble single precision", |b| {
         b.iter(|| {
-            make_helmholtz_evaluator(sources.view(), targets.view(), wavenumber)
-                .assemble_in_place(black_box(result.view_mut()), ThreadingType::Parallel);
+            c32::assemble_kernel_in_place(sources.view(), targets.view(), black_box(result.view_mut()), KernelType::Helmholtz(wavenumber), num_cpus::get());
         })
     });
 }
@@ -196,7 +173,7 @@ fn benchmark_helmholtz_assemble_double_precision(c: &mut Criterion) {
     let nsources = 20000;
     let ntargets = 20000;
 
-    let wavenumber = Complex::new(2.5, 0.0);
+    let wavenumber = c64::new(2.5, 0.0);
 
     type MyType = f64;
 
@@ -204,15 +181,14 @@ fn benchmark_helmholtz_assemble_double_precision(c: &mut Criterion) {
 
     let mut sources = ndarray::Array2::<MyType>::zeros((3, nsources));
     let mut targets = ndarray::Array2::<MyType>::zeros((3, ntargets));
-    let mut result = ndarray::Array2::<Complex<MyType>>::zeros((ntargets, nsources));
+    let mut result = ndarray::Array2::<c64>::zeros((ntargets, nsources));
 
     sources.map_inplace(|item| *item = rng.gen::<MyType>());
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
 
     c.bench_function("helmholtz assemble double precision", |b| {
         b.iter(|| {
-            make_helmholtz_evaluator(sources.view(), targets.view(), wavenumber)
-                .assemble_in_place(black_box(result.view_mut()), ThreadingType::Parallel);
+            c64::assemble_kernel_in_place(sources.view(), targets.view(), black_box(result.view_mut()), KernelType::Helmholtz(wavenumber), num_cpus::get());
         })
     });
 }
@@ -224,14 +200,14 @@ fn benchmark_helmholtz_evaluate_values_double_precision(c: &mut Criterion) {
 
     type MyType = f64;
 
-    let wavenumber = Complex::new(2.5, 0.0);
+    let wavenumber = c64::new(2.5, 0.0);
 
     let mut rng = rand::thread_rng();
 
     let mut sources = ndarray::Array2::<MyType>::zeros((3, nsources));
     let mut targets = ndarray::Array2::<MyType>::zeros((3, ntargets));
-    let mut charges = ndarray::Array2::<Complex<MyType>>::zeros((ncharge_vecs, nsources));
-    let mut result = ndarray::Array3::<Complex<MyType>>::zeros((ncharge_vecs, ntargets, 1));
+    let mut charges = ndarray::Array2::<c64>::zeros((ncharge_vecs, nsources));
+    let mut result = ndarray::Array3::<c64>::zeros((ncharge_vecs, ntargets, 1));
 
     sources.map_inplace(|item| *item = rng.gen::<MyType>());
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
@@ -242,12 +218,7 @@ fn benchmark_helmholtz_evaluate_values_double_precision(c: &mut Criterion) {
 
     c.bench_function("helmholtz evaluate values double precision", |b| {
         b.iter(|| {
-            make_helmholtz_evaluator(sources.view(), targets.view(), wavenumber).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::Value,
-                ThreadingType::Parallel,
-            );
+            c64::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Helmholtz(wavenumber), EvalMode::Value, num_cpus::get());
         })
     });
 }
@@ -259,14 +230,14 @@ fn benchmark_helmholtz_evaluate_values_single_precision(c: &mut Criterion) {
 
     type MyType = f32;
 
-    let wavenumber = Complex::new(2.5, 0.0);
+    let wavenumber = c64::new(2.5, 0.0);
 
     let mut rng = rand::thread_rng();
 
     let mut sources = ndarray::Array2::<MyType>::zeros((3, nsources));
     let mut targets = ndarray::Array2::<MyType>::zeros((3, ntargets));
-    let mut charges = ndarray::Array2::<Complex<MyType>>::zeros((ncharge_vecs, nsources));
-    let mut result = ndarray::Array3::<Complex<MyType>>::zeros((ncharge_vecs, ntargets, 1));
+    let mut charges = ndarray::Array2::<c32>::zeros((ncharge_vecs, nsources));
+    let mut result = ndarray::Array3::<c32>::zeros((ncharge_vecs, ntargets, 1));
 
     sources.map_inplace(|item| *item = rng.gen::<MyType>());
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
@@ -277,12 +248,7 @@ fn benchmark_helmholtz_evaluate_values_single_precision(c: &mut Criterion) {
 
     c.bench_function("helmholtz evaluate values single precision", |b| {
         b.iter(|| {
-            make_helmholtz_evaluator(sources.view(), targets.view(), wavenumber).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::Value,
-                ThreadingType::Parallel,
-            );
+            c32::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Helmholtz(wavenumber), EvalMode::Value, num_cpus::get());
         })
     });
 }
@@ -294,14 +260,14 @@ fn benchmark_helmholtz_evaluate_values_and_derivs_double_precision(c: &mut Crite
 
     type MyType = f64;
 
-    let wavenumber = Complex::new(2.5, 0.0);
+    let wavenumber = c64::new(2.5, 0.0);
 
     let mut rng = rand::thread_rng();
 
     let mut sources = ndarray::Array2::<MyType>::zeros((3, nsources));
     let mut targets = ndarray::Array2::<MyType>::zeros((3, ntargets));
-    let mut charges = ndarray::Array2::<Complex<MyType>>::zeros((ncharge_vecs, nsources));
-    let mut result = ndarray::Array3::<Complex<MyType>>::zeros((ncharge_vecs, ntargets, 4));
+    let mut charges = ndarray::Array2::<c64>::zeros((ncharge_vecs, nsources));
+    let mut result = ndarray::Array3::<c64>::zeros((ncharge_vecs, ntargets, 4));
 
     sources.map_inplace(|item| *item = rng.gen::<MyType>());
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
@@ -310,19 +276,16 @@ fn benchmark_helmholtz_evaluate_values_and_derivs_double_precision(c: &mut Crite
         item.im = rng.gen::<MyType>();
     });
 
-    c.bench_function("helmholtz evaluate values and derivs double precision", |b| {
-        b.iter(|| {
-            make_helmholtz_evaluator(sources.view(), targets.view(), wavenumber).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::ValueGrad,
-                ThreadingType::Parallel,
-            );
-        })
-    });
+    c.bench_function(
+        "helmholtz evaluate values and derivs double precision",
+        |b| {
+            b.iter(|| {
+                c64::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Helmholtz(wavenumber), EvalMode::ValueGrad, num_cpus::get());
+            })
+        },
+    );
 }
 
-
 fn benchmark_helmholtz_evaluate_values_and_derivs_single_precision(c: &mut Criterion) {
     let nsources = 20000;
     let ntargets = 20000;
@@ -330,14 +293,14 @@ fn benchmark_helmholtz_evaluate_values_and_derivs_single_precision(c: &mut Crite
 
     type MyType = f32;
 
-    let wavenumber = Complex::new(2.5, 0.0);
+    let wavenumber = c64::new(2.5, 0.0);
 
     let mut rng = rand::thread_rng();
 
     let mut sources = ndarray::Array2::<MyType>::zeros((3, nsources));
     let mut targets = ndarray::Array2::<MyType>::zeros((3, ntargets));
-    let mut charges = ndarray::Array2::<Complex<MyType>>::zeros((ncharge_vecs, nsources));
-    let mut result = ndarray::Array3::<Complex<MyType>>::zeros((ncharge_vecs, ntargets, 4));
+    let mut charges = ndarray::Array2::<c32>::zeros((ncharge_vecs, nsources));
+    let mut result = ndarray::Array3::<c32>::zeros((ncharge_vecs, ntargets, 4));
 
     sources.map_inplace(|item| *item = rng.gen::<MyType>());
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
@@ -346,19 +309,16 @@ fn benchmark_helmholtz_evaluate_values_and_derivs_single_precision(c: &mut Crite
         item.im = rng.gen::<MyType>();
     });
 
-    c.bench_function("helmholtz evaluate values and derivs single precision", |b| {
-        b.iter(|| {
-            make_helmholtz_evaluator(sources.view(), targets.view(), wavenumber).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::ValueGrad,
-                ThreadingType::Parallel,
-            );
-        })
-    });
+    c.bench_function(
+        "helmholtz evaluate values and derivs single precision",
+        |b| {
+            b.iter(|| {
+                c32::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::Helmholtz(wavenumber), EvalMode::ValueGrad, num_cpus::get());
+            })
+        },
+    );
 }
 
-
 fn benchmark_modified_helmholtz_assemble_double_precision(c: &mut Criterion) {
     let nsources = 20000;
     let ntargets = 20000;
@@ -376,8 +336,7 @@ fn benchmark_modified_helmholtz_assemble_double_precision(c: &mut Criterion) {
 
     c.bench_function("modified helmholtz assemble double precision", |b| {
         b.iter(|| {
-            make_modified_helmholtz_evaluator(sources.view(), targets.view(), omega)
-                .assemble_in_place(black_box(result.view_mut()), ThreadingType::Parallel);
+            f64::assemble_kernel_in_place(sources.view(), targets.view(), black_box(result.view_mut()), KernelType::ModifiedHelmholtz(omega), num_cpus::get());
         })
     });
 }
@@ -399,8 +358,7 @@ fn benchmark_modified_helmholtz_assemble_single_precision(c: &mut Criterion) {
 
     c.bench_function("modified helmholtz assemble single precision", |b| {
         b.iter(|| {
-            make_modified_helmholtz_evaluator(sources.view(), targets.view(), omega)
-                .assemble_in_place(black_box(result.view_mut()), ThreadingType::Parallel);
+            f32::assemble_kernel_in_place(sources.view(), targets.view(), black_box(result.view_mut()), KernelType::ModifiedHelmholtz(omega), num_cpus::get());
         })
     });
 }
@@ -427,12 +385,7 @@ fn benchmark_modified_helmholtz_evaluate_values_double_precision(c: &mut Criteri
 
     c.bench_function("modified helmholtz evaluate values double precision", |b| {
         b.iter(|| {
-            make_modified_helmholtz_evaluator(sources.view(), targets.view(), omega).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::Value,
-                ThreadingType::Parallel,
-            );
+            f64::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::ModifiedHelmholtz(omega), EvalMode::Value, num_cpus::get());
         })
     });
 }
@@ -459,12 +412,7 @@ fn benchmark_modified_helmholtz_evaluate_values_single_precision(c: &mut Criteri
 
     c.bench_function("modified helmholtz evaluate values single precision", |b| {
         b.iter(|| {
-            make_modified_helmholtz_evaluator(sources.view(), targets.view(), omega).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::Value,
-                ThreadingType::Parallel,
-            );
+            f32::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::ModifiedHelmholtz(omega), EvalMode::Value, num_cpus::get());
         })
     });
 }
@@ -489,16 +437,14 @@ fn benchmark_modified_helmholtz_evaluate_values_and_derivs_double_precision(c: &
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
     charges.map_inplace(|item| *item = rng.gen::<MyType>());
 
-    c.bench_function("modified helmholtz evaluate values and derivs double precision", |b| {
-        b.iter(|| {
-            make_modified_helmholtz_evaluator(sources.view(), targets.view(), omega).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::ValueGrad,
-                ThreadingType::Parallel,
-            );
-        })
-    });
+    c.bench_function(
+        "modified helmholtz evaluate values and derivs double precision",
+        |b| {
+            b.iter(|| {
+                f64::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::ModifiedHelmholtz(omega), EvalMode::ValueGrad, num_cpus::get());
+            })
+        },
+    );
 }
 
 fn benchmark_modified_helmholtz_evaluate_values_and_derivs_single_precision(c: &mut Criterion) {
@@ -521,42 +467,38 @@ fn benchmark_modified_helmholtz_evaluate_values_and_derivs_single_precision(c: &
     targets.map_inplace(|item| *item = rng.gen::<MyType>());
     charges.map_inplace(|item| *item = rng.gen::<MyType>());
 
-    c.bench_function("modified helmholtz evaluate values and derivs single precision", |b| {
-        b.iter(|| {
-            make_modified_helmholtz_evaluator(sources.view(), targets.view(), omega).evaluate_in_place(
-                charges.view(),
-                black_box(result.view_mut()),
-                &EvalMode::ValueGrad,
-                ThreadingType::Parallel,
-            );
-        })
-    });
+    c.bench_function(
+        "modified helmholtz evaluate values and derivs single precision",
+        |b| {
+            b.iter(|| {
+                f32::evaluate_kernel_in_place(sources.view(), targets.view(), charges.view(), black_box(result.view_mut()), KernelType::ModifiedHelmholtz(omega), EvalMode::ValueGrad, num_cpus::get());
+            })
+        },
+    );
 }
 
-
-
 criterion_group! {
-    name = benches;
-    config = Criterion::default().sample_size(30).measurement_time(std::time::Duration::from_secs(10));
-    targets = benchmark_laplace_assemble_single_precision,
-              benchmark_laplace_assemble_double_precision,
-              benchmark_laplace_evalaute_values_single_precision,
-              benchmark_laplace_evaluate_values_double_precision,
-              benchmark_laplace_evaluate_values_and_derivs_single_precision,
-              benchmark_laplace_evaluate_values_and_derivs_double_precision,
-              benchmark_helmholtz_assemble_single_precision,
-              benchmark_helmholtz_assemble_double_precision,
-              benchmark_helmholtz_evaluate_values_double_precision,
-              benchmark_helmholtz_evaluate_values_single_precision,
-              benchmark_helmholtz_evaluate_values_and_derivs_double_precision,
-              benchmark_helmholtz_evaluate_values_and_derivs_single_precision,
-              benchmark_modified_helmholtz_assemble_single_precision,
-              benchmark_modified_helmholtz_assemble_double_precision,
-              benchmark_modified_helmholtz_evaluate_values_double_precision,
-              benchmark_modified_helmholtz_evaluate_values_single_precision,
-              benchmark_modified_helmholtz_evaluate_values_and_derivs_double_precision,
-              benchmark_modified_helmholtz_evaluate_values_and_derivs_single_precision,
+name = benches;
+config = Criterion::default().sample_size(30).measurement_time(std::time::Duration::from_secs(10));
+targets = benchmark_laplace_assemble_single_precision,
+          benchmark_laplace_assemble_double_precision,
+          benchmark_laplace_evalaute_values_single_precision,
+          benchmark_laplace_evaluate_values_double_precision,
+          benchmark_laplace_evaluate_values_and_derivs_single_precision,
+          benchmark_laplace_evaluate_values_and_derivs_double_precision,
+          benchmark_helmholtz_assemble_single_precision,
+          benchmark_helmholtz_assemble_double_precision,
+          benchmark_helmholtz_evaluate_values_double_precision,
+          benchmark_helmholtz_evaluate_values_single_precision,
+          benchmark_helmholtz_evaluate_values_and_derivs_double_precision,
+          benchmark_helmholtz_evaluate_values_and_derivs_single_precision,
+          benchmark_modified_helmholtz_assemble_single_precision,
+          benchmark_modified_helmholtz_assemble_double_precision,
+          benchmark_modified_helmholtz_evaluate_values_double_precision,
+          benchmark_modified_helmholtz_evaluate_values_single_precision,
+          benchmark_modified_helmholtz_evaluate_values_and_derivs_double_precision,
+          benchmark_modified_helmholtz_evaluate_values_and_derivs_single_precision,
 
 
-            }
+        }
 criterion_main!(benches);
