@@ -2,10 +2,14 @@
 import numpy as np
 import pytest
 
+import os
 
-@pytest.mark.parametrize("parallel", [True, False])
+CPU_COUNT = os.cpu_count()
+
+
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
 @pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 5e-6)])
-def test_laplace_assemble(dtype, rtol, parallel):
+def test_laplace_assemble(dtype, rtol, num_threads):
     """Test the Laplace kernel."""
     from rusty_green_kernel import assemble_laplace_kernel
 
@@ -20,7 +24,9 @@ def test_laplace_assemble(dtype, rtol, parallel):
     sources = rng.random((3, nsources), dtype=dtype)
     sources[:, 0] = targets[:, 0]  # Test what happens if source = target
 
-    actual = assemble_laplace_kernel(sources, targets, dtype=dtype, parallel=parallel)
+    actual = assemble_laplace_kernel(
+        sources, targets, dtype=dtype, num_threads=num_threads
+    )
 
     # Calculate expected result
 
@@ -44,9 +50,9 @@ def test_laplace_assemble(dtype, rtol, parallel):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
-@pytest.mark.parametrize("parallel", [True, False])
-@pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 5e-6)])
-def test_laplace_evaluate_only_values(dtype, rtol, parallel):
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
+@pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 1e-4)])
+def test_laplace_evaluate_only_values(dtype, rtol, num_threads):
     """Test the Laplace kernel."""
     from rusty_green_kernel import evaluate_laplace_kernel
 
@@ -64,7 +70,7 @@ def test_laplace_evaluate_only_values(dtype, rtol, parallel):
     charges = rng.random((ncharge_vecs, nsources), dtype=dtype)
 
     actual = evaluate_laplace_kernel(
-        sources, targets, charges, dtype=dtype, parallel=parallel
+        sources, targets, charges, dtype=dtype, num_threads=num_threads
     )
 
     # Calculate expected result
@@ -91,9 +97,9 @@ def test_laplace_evaluate_only_values(dtype, rtol, parallel):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
-@pytest.mark.parametrize("parallel", [True, False])
-@pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 5e-6)])
-def test_laplace_evaluate_values_and_deriv(dtype, rtol, parallel):
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
+@pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 1e-4)])
+def test_laplace_evaluate_values_and_deriv(dtype, rtol, num_threads):
     """Test the Laplace kernel."""
     from rusty_green_kernel import evaluate_laplace_kernel
 
@@ -111,7 +117,12 @@ def test_laplace_evaluate_values_and_deriv(dtype, rtol, parallel):
     charges = rng.random((ncharge_vecs, nsources), dtype=dtype)
 
     actual = evaluate_laplace_kernel(
-        sources, targets, charges, dtype=dtype, return_gradients=True, parallel=parallel
+        sources,
+        targets,
+        charges,
+        dtype=dtype,
+        return_gradients=True,
+        num_threads=num_threads,
     )
 
     # Calculate expected result
@@ -138,9 +149,9 @@ def test_laplace_evaluate_values_and_deriv(dtype, rtol, parallel):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
-@pytest.mark.parametrize("parallel", [True, False])
-@pytest.mark.parametrize("dtype,rtol", [(np.complex128, 1e-14), (np.complex64, 5e-6)])
-def test_helmholtz_assemble(dtype, rtol, parallel):
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
+@pytest.mark.parametrize("dtype,rtol", [(np.complex128, 1e-14), (np.complex64, 1E-4)])
+def test_helmholtz_assemble(dtype, rtol, num_threads):
     """Test the Laplace kernel."""
     from rusty_green_kernel import assemble_helmholtz_kernel
 
@@ -165,7 +176,7 @@ def test_helmholtz_assemble(dtype, rtol, parallel):
     sources[:, 0] = targets[:, 0]  # Test what happens if source = target
 
     actual = assemble_helmholtz_kernel(
-        sources, targets, wavenumber, dtype=dtype, parallel=parallel
+        sources, targets, wavenumber, dtype=dtype, num_threads=num_threads
     )
 
     # Calculate expected result
@@ -188,8 +199,9 @@ def test_helmholtz_assemble(dtype, rtol, parallel):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
 @pytest.mark.parametrize("dtype,rtol", [(np.complex128, 1e-14), (np.complex64, 5e-6)])
-def test_helmholtz_evaluate_only_values(dtype, rtol):
+def test_helmholtz_evaluate_only_values(dtype, rtol, num_threads):
     """Test the Laplace kernel."""
     from rusty_green_kernel import evaluate_helmholtz_kernel
 
@@ -218,7 +230,7 @@ def test_helmholtz_evaluate_only_values(dtype, rtol):
     )
 
     actual = evaluate_helmholtz_kernel(
-        sources, targets, charges, wavenumber, dtype=dtype, parallel=False
+        sources, targets, charges, wavenumber, dtype=dtype, num_threads=num_threads
     )
 
     # Calculate expected result
@@ -243,9 +255,9 @@ def test_helmholtz_evaluate_only_values(dtype, rtol):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
-@pytest.mark.parametrize("parallel", [True, False])
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
 @pytest.mark.parametrize("dtype,rtol", [(np.complex128, 1e-14), (np.complex64, 5e-6)])
-def test_helmholtz_evaluate_values_and_deriv(dtype, rtol, parallel):
+def test_helmholtz_evaluate_values_and_deriv(dtype, rtol, num_threads):
     """Test the Laplace kernel."""
     from rusty_green_kernel import evaluate_helmholtz_kernel
 
@@ -280,7 +292,7 @@ def test_helmholtz_evaluate_values_and_deriv(dtype, rtol, parallel):
         wavenumber,
         dtype=dtype,
         return_gradients=True,
-        parallel=parallel,
+        num_threads=num_threads,
     )
 
     # Calculate expected result
@@ -312,9 +324,9 @@ def test_helmholtz_evaluate_values_and_deriv(dtype, rtol, parallel):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
-@pytest.mark.parametrize("parallel", [True, False])
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
 @pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 5e-6)])
-def test_modified_helmholtz_assemble(dtype, rtol, parallel):
+def test_modified_helmholtz_assemble(dtype, rtol, num_threads):
     """Test the modified Helmholtz kernel."""
     from rusty_green_kernel import assemble_modified_helmholtz_kernel
 
@@ -332,7 +344,7 @@ def test_modified_helmholtz_assemble(dtype, rtol, parallel):
     sources[:, 0] = targets[:, 0]  # Test what happens if source = target
 
     actual = assemble_modified_helmholtz_kernel(
-        sources, targets, omega, dtype=dtype, parallel=parallel
+        sources, targets, omega, dtype=dtype, num_threads=num_threads
     )
 
     # Calculate expected result
@@ -356,9 +368,9 @@ def test_modified_helmholtz_assemble(dtype, rtol, parallel):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
-@pytest.mark.parametrize("parallel", [True, False])
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
 @pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 5e-6)])
-def test_modified_helmholtz_evaluate_only_values(dtype, rtol, parallel):
+def test_modified_helmholtz_evaluate_only_values(dtype, rtol, num_threads):
     """Test the modified Helmholtz kernel."""
     from rusty_green_kernel import evaluate_modified_helmholtz_kernel
 
@@ -378,7 +390,7 @@ def test_modified_helmholtz_evaluate_only_values(dtype, rtol, parallel):
     charges = rng.random((ncharge_vecs, nsources), dtype=dtype)
 
     actual = evaluate_modified_helmholtz_kernel(
-        sources, targets, charges, omega, dtype=dtype, parallel=parallel
+        sources, targets, charges, omega, dtype=dtype, num_threads=num_threads
     )
 
     # Calculate expected result
@@ -404,9 +416,9 @@ def test_modified_helmholtz_evaluate_only_values(dtype, rtol, parallel):
     np.testing.assert_allclose(actual, expected, rtol=rtol)
 
 
-@pytest.mark.parametrize("parallel", [True, False])
+@pytest.mark.parametrize("num_threads", [1, CPU_COUNT])
 @pytest.mark.parametrize("dtype,rtol", [(np.float64, 1e-14), (np.float32, 5e-6)])
-def test_modified_helmholtz_evaluate_values_and_deriv(dtype, rtol, parallel):
+def test_modified_helmholtz_evaluate_values_and_deriv(dtype, rtol, num_threads):
     """Test the modified Helmholtz kernel."""
     from rusty_green_kernel import evaluate_modified_helmholtz_kernel
 
@@ -432,7 +444,7 @@ def test_modified_helmholtz_evaluate_values_and_deriv(dtype, rtol, parallel):
         omega,
         dtype=dtype,
         return_gradients=True,
-        parallel=parallel,
+        num_threads=num_threads,
     )
 
     # Calculate expected result
@@ -635,6 +647,7 @@ def test_helmholtz_at_zero_agrees_with_laplace():
 
     np.testing.assert_allclose(values_helmholtz, values_laplace, rtol=1E-14)
 
+
 def test_helmholtz_imaginary_wavenumber_agrees_with_modified_helmholtz():
     """Test if Helmholtz with wavenumber 0 agrees with Laplace."""
     from rusty_green_kernel import evaluate_helmholtz_kernel
@@ -668,4 +681,4 @@ def test_helmholtz_imaginary_wavenumber_agrees_with_modified_helmholtz():
         sources, targets, np.imag(charges), np.imag(wavenumber), return_gradients=True
     )
 
-    np.testing.assert_allclose(values_helmholtz, values_modified_helmholtz, rtol=1E-14)
\ No newline at end of file
+    np.testing.assert_allclose(values_helmholtz, values_modified_helmholtz, rtol=1E-14)
