@@ -1,26 +1,20 @@
 //! This module defines C API function to access all assembly and evaluation routines.
 
+use crate::*;
 use ndarray;
-use num::complex::Complex;
-use rusty_kernel_tools::ThreadingType;
-
-use crate::RealDirectEvaluator;
-use crate::ComplexDirectEvaluator;
-use crate::make_laplace_evaluator;
-use crate::make_helmholtz_evaluator;
-use crate::make_modified_helmholtz_evaluator;
+use ndarray_linalg::{c32, c64};
 
 /// Assemble the Laplace kernel (double precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `result_ptr` - Pointer to an existing `(ntargets, nsources)` array that stores the result.
 /// * `nsources`   - Number of sources.
 /// * `ntargets`   - Number of targets.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn assemble_laplace_kernel_f64(
     source_ptr: *const f64,
@@ -28,33 +22,27 @@ pub extern "C" fn assemble_laplace_kernel_f64(
     result_ptr: *mut f64,
     nsources: usize,
     ntargets: usize,
-    parallel: bool,
+    num_threads: usize,
 ) {
-
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
     let sources = unsafe { ndarray::ArrayView2::from_shape_ptr((3, nsources), source_ptr) };
     let result =
         unsafe { ndarray::ArrayViewMut2::from_shape_ptr((ntargets, nsources), result_ptr) };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
-    make_laplace_evaluator(sources, targets).assemble_in_place(result, threading_type);
+    f64::assemble_kernel_in_place(sources, targets, result, KernelType::Laplace, num_threads);
 }
 
 /// Assemble the Laplace kernel (single precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `result_ptr` - Pointer to an existing `(ntargets, nsources)` array that stores the result.
 /// * `nsources`   - Number of sources.
 /// * `ntargets`   - Number of targets.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn assemble_laplace_kernel_f32(
     source_ptr: *const f32,
@@ -62,26 +50,21 @@ pub extern "C" fn assemble_laplace_kernel_f32(
     result_ptr: *mut f32,
     nsources: usize,
     ntargets: usize,
-    parallel: bool,
+    num_threads: usize,
 ) {
-
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
     let sources = unsafe { ndarray::ArrayView2::from_shape_ptr((3, nsources), source_ptr) };
     let result =
         unsafe { ndarray::ArrayViewMut2::from_shape_ptr((ntargets, nsources), result_ptr) };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
-    make_laplace_evaluator(sources, targets).assemble_in_place(result, threading_type);
+    f32::assemble_kernel_in_place(sources, targets, result, KernelType::Laplace, num_threads);
 }
+
 /// Evaluate the Laplace potential sum (double precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `charge_ptr` - Pointer to a `(ncharge_vecs, nsources)` array of `ncharge_vecs` charge vectors.
@@ -91,7 +74,7 @@ pub extern "C" fn assemble_laplace_kernel_f32(
 /// * `ntargets`   - Number of targets.
 /// * `ncharge_vecs` - Number of charge vectors.
 /// * `return_gradients` - If true return also the gradients.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn evaluate_laplace_kernel_f64(
     source_ptr: *const f64,
@@ -102,20 +85,15 @@ pub extern "C" fn evaluate_laplace_kernel_f64(
     ntargets: usize,
     ncharge_vecs: usize,
     return_gradients: bool,
-    parallel: bool,
+    num_threads: usize,
 ) {
-    use crate::kernels::EvalMode;
+    let kernel_type = KernelType::Laplace;
 
     let eval_mode = match return_gradients {
         true => EvalMode::ValueGrad,
         false => EvalMode::Value,
     };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
     let ncols: usize = match eval_mode {
         EvalMode::Value => 1,
         EvalMode::ValueGrad => 4,
@@ -129,19 +107,22 @@ pub extern "C" fn evaluate_laplace_kernel_f64(
         ndarray::ArrayViewMut3::from_shape_ptr((ncharge_vecs, ntargets, ncols), result_ptr)
     };
 
-    make_laplace_evaluator(sources, targets).evaluate_in_place(
+    f64::evaluate_kernel_in_place(
+        sources,
+        targets,
         charges,
         result,
-        &eval_mode,
-        threading_type,
+        kernel_type,
+        eval_mode,
+        num_threads,
     );
 }
 
 /// Evaluate the Laplace potential sum (single precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `charge_ptr` - Pointer to a `(ncharge_vecs, nsources)` array of `ncharge_vecs` charge vectors.
@@ -151,7 +132,7 @@ pub extern "C" fn evaluate_laplace_kernel_f64(
 /// * `ntargets`   - Number of targets.
 /// * `ncharge_vecs` - Number of charge vectors.
 /// * `return_gradients` - If true return also the gradients.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn evaluate_laplace_kernel_f32(
     source_ptr: *const f32,
@@ -162,20 +143,15 @@ pub extern "C" fn evaluate_laplace_kernel_f32(
     ntargets: usize,
     ncharge_vecs: usize,
     return_gradients: bool,
-    parallel: bool,
+    num_threads: usize,
 ) {
-    use crate::kernels::EvalMode;
+    let kernel_type = KernelType::Laplace;
 
     let eval_mode = match return_gradients {
         true => EvalMode::ValueGrad,
         false => EvalMode::Value,
     };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
     let ncols: usize = match eval_mode {
         EvalMode::Value => 1,
         EvalMode::ValueGrad => 4,
@@ -189,19 +165,22 @@ pub extern "C" fn evaluate_laplace_kernel_f32(
         ndarray::ArrayViewMut3::from_shape_ptr((ncharge_vecs, ntargets, ncols), result_ptr)
     };
 
-    make_laplace_evaluator(sources, targets).evaluate_in_place(
+    f32::evaluate_kernel_in_place(
+        sources,
+        targets,
         charges,
         result,
-        &eval_mode,
-        threading_type,
+        kernel_type,
+        eval_mode,
+        num_threads,
     );
 }
 
 /// Assemble the Helmholtz kernel (double precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `result_ptr` - Pointer to an existing `(ntargets,  2 * nsources)` array that stores the result
@@ -210,7 +189,7 @@ pub extern "C" fn evaluate_laplace_kernel_f32(
 /// * `wavenumber_imag` - Imaginary part of the wavenumber parameter.
 /// * `nsources`   - Number of sources.
 /// * `ntargets`   - Number of targets.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn assemble_helmholtz_kernel_f64(
     source_ptr: *const f64,
@@ -220,7 +199,7 @@ pub extern "C" fn assemble_helmholtz_kernel_f64(
     wavenumber_imag: f64,
     nsources: usize,
     ntargets: usize,
-    parallel: bool,
+    num_threads: usize,
 ) {
 
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
@@ -228,25 +207,23 @@ pub extern "C" fn assemble_helmholtz_kernel_f64(
     let result = unsafe {
         ndarray::ArrayViewMut2::from_shape_ptr(
             (ntargets, nsources),
-            result_ptr as *mut Complex<f64>,
+            result_ptr as *mut c64,
         )
     };
-    let wavenumber = Complex::<f64>::new(wavenumber_real, wavenumber_imag);
+    let wavenumber = c64::new(wavenumber_real, wavenumber_imag);
+    let kernel_type = KernelType::Helmholtz(wavenumber);
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
 
-    make_helmholtz_evaluator(sources, targets, wavenumber)
-        .assemble_in_place(result, threading_type);
+    c64::assemble_kernel_in_place(sources, targets, result, kernel_type, num_threads);
+
+    
 }
 
 /// Assemble the Helmholtz kernel (single precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `result_ptr` - Pointer to an existing `(ntargets, 2 * nsources)` array that stores the result
@@ -255,7 +232,7 @@ pub extern "C" fn assemble_helmholtz_kernel_f64(
 /// * `wavenumber_imag` - Imaginary part of the wavenumber parameter.
 /// * `nsources`   - Number of sources.
 /// * `ntargets`   - Number of targets.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn assemble_helmholtz_kernel_f32(
     source_ptr: *const f32,
@@ -265,7 +242,7 @@ pub extern "C" fn assemble_helmholtz_kernel_f32(
     wavenumber_imag: f64,
     nsources: usize,
     ntargets: usize,
-    parallel: bool,
+    num_threads: usize,
 ) {
 
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
@@ -273,25 +250,21 @@ pub extern "C" fn assemble_helmholtz_kernel_f32(
     let result = unsafe {
         ndarray::ArrayViewMut2::from_shape_ptr(
             (ntargets, nsources),
-            result_ptr as *mut Complex<f32>,
+            result_ptr as *mut c32,
         )
     };
-    let wavenumber = Complex::<f64>::new(wavenumber_real, wavenumber_imag);
+    let wavenumber = c64::new(wavenumber_real, wavenumber_imag);
+    let kernel_type = KernelType::Helmholtz(wavenumber);
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
 
-    make_helmholtz_evaluator(sources, targets, wavenumber)
-        .assemble_in_place(result, threading_type);
+    c32::assemble_kernel_in_place(sources, targets, result, kernel_type, num_threads);
 }
 
 /// Evaluate the Helmholtz potential sum (double precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `charge_ptr` - Pointer to a `(ncharge_vecs, nsources)` array of `ncharge_vecs` charge vectors.
@@ -304,7 +277,7 @@ pub extern "C" fn assemble_helmholtz_kernel_f32(
 /// * `ntargets`   - Number of targets.
 /// * `ncharge_vecs` - Number of charge vectors.
 /// * `return_gradients` - If true return also the gradients.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn evaluate_helmholtz_kernel_f64(
     source_ptr: *const f64,
@@ -317,55 +290,54 @@ pub extern "C" fn evaluate_helmholtz_kernel_f64(
     ntargets: usize,
     ncharge_vecs: usize,
     return_gradients: bool,
-    parallel: bool,
+    num_threads: usize,
 ) {
-    use crate::kernels::EvalMode;
 
     let eval_mode = match return_gradients {
         true => EvalMode::ValueGrad,
         false => EvalMode::Value,
     };
-
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
+        
     let ncols: usize = match eval_mode {
         EvalMode::Value => 1,
         EvalMode::ValueGrad => 4,
     };
-    let wavenumber = Complex::<f64>::new(wavenumber_real, wavenumber_imag);
+    let wavenumber = c64::new(wavenumber_real, wavenumber_imag);
 
+    let kernel_type = KernelType::Helmholtz(wavenumber);
+    
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
     let sources = unsafe { ndarray::ArrayView2::from_shape_ptr((3, nsources), source_ptr) };
     let charges = unsafe {
         ndarray::ArrayView2::from_shape_ptr(
             (ncharge_vecs, nsources),
-            charge_ptr as *mut Complex<f64>,
+            charge_ptr as *mut c64,
         )
     };
-
+    
     let result = unsafe {
         ndarray::ArrayViewMut3::from_shape_ptr(
             (ncharge_vecs, ntargets, ncols),
-            result_ptr as *mut Complex<f64>,
+            result_ptr as *mut c64,
         )
     };
-
-    make_helmholtz_evaluator(sources, targets, wavenumber).evaluate_in_place(
+    
+    c64::evaluate_kernel_in_place(
+        sources,
+        targets,
         charges,
         result,
-        &eval_mode,
-        threading_type,
+        kernel_type,
+        eval_mode,
+        num_threads,
     );
 }
 
 /// Evaluate the Helmholtz potential sum (single precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `charge_ptr` - Pointer to a `(ncharge_vecs, nsources)` array of `ncharge_vecs` charge vectors.
@@ -378,7 +350,7 @@ pub extern "C" fn evaluate_helmholtz_kernel_f64(
 /// * `ntargets`   - Number of targets.
 /// * `ncharge_vecs` - Number of charge vectors.
 /// * `return_gradients` - If true return also the gradients.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn evaluate_helmholtz_kernel_f32(
     source_ptr: *const f32,
@@ -391,61 +363,61 @@ pub extern "C" fn evaluate_helmholtz_kernel_f32(
     ntargets: usize,
     ncharge_vecs: usize,
     return_gradients: bool,
-    parallel: bool,
+    num_threads: usize,
 ) {
-    use crate::kernels::EvalMode;
 
     let eval_mode = match return_gradients {
         true => EvalMode::ValueGrad,
         false => EvalMode::Value,
     };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
     let ncols: usize = match eval_mode {
         EvalMode::Value => 1,
         EvalMode::ValueGrad => 4,
     };
-    let wavenumber = Complex::<f64>::new(wavenumber_real, wavenumber_imag);
+    let wavenumber = c64::new(wavenumber_real, wavenumber_imag);
 
+    let kernel_type = KernelType::Helmholtz(wavenumber);
+    
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
     let sources = unsafe { ndarray::ArrayView2::from_shape_ptr((3, nsources), source_ptr) };
     let charges = unsafe {
         ndarray::ArrayView2::from_shape_ptr(
             (ncharge_vecs, nsources),
-            charge_ptr as *mut Complex<f32>,
+            charge_ptr as *mut c32,
         )
     };
+    
     let result = unsafe {
         ndarray::ArrayViewMut3::from_shape_ptr(
             (ncharge_vecs, ntargets, ncols),
-            result_ptr as *mut Complex<f32>,
+            result_ptr as *mut c32,
         )
     };
-
-    make_helmholtz_evaluator(sources, targets, wavenumber).evaluate_in_place(
+    
+    c32::evaluate_kernel_in_place(
+        sources,
+        targets,
         charges,
         result,
-        &eval_mode,
-        threading_type,
+        kernel_type,
+        eval_mode,
+        num_threads,
     );
 }
 
 /// Assemble the modified Helmholtz kernel (double precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `result_ptr` - Pointer to an existing `(ntargets, nsources)` array that stores the result.
 /// * `omega`      - The omega parameter of the kernel.
 /// * `nsources`   - Number of sources.
 /// * `ntargets`   - Number of targets.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn assemble_modified_helmholtz_kernel_f64(
     source_ptr: *const f64,
@@ -454,7 +426,7 @@ pub extern "C" fn assemble_modified_helmholtz_kernel_f64(
     omega: f64,
     nsources: usize,
     ntargets: usize,
-    parallel: bool,
+    num_threads: usize,
 ) {
 
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
@@ -462,26 +434,21 @@ pub extern "C" fn assemble_modified_helmholtz_kernel_f64(
     let result =
         unsafe { ndarray::ArrayViewMut2::from_shape_ptr((ntargets, nsources), result_ptr) };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
-    make_modified_helmholtz_evaluator(sources, targets, omega).assemble_in_place(result, threading_type);
+    f64::assemble_kernel_in_place(sources, targets, result, KernelType::ModifiedHelmholtz(omega), num_threads);
 }
 
 /// Assemble the modified Helmholtz kernel (single precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `result_ptr` - Pointer to an existing `(ntargets, nsources)` array that stores the result.
 /// * `omega`      - The omega parameter of the kernel.
 /// * `nsources`   - Number of sources.
 /// * `ntargets`   - Number of targets.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn assemble_modified_helmholtz_kernel_f32(
     source_ptr: *const f32,
@@ -490,7 +457,7 @@ pub extern "C" fn assemble_modified_helmholtz_kernel_f32(
     omega: f64,
     nsources: usize,
     ntargets: usize,
-    parallel: bool,
+    num_threads: usize,
 ) {
 
     let targets = unsafe { ndarray::ArrayView2::from_shape_ptr((3, ntargets), target_ptr) };
@@ -498,19 +465,14 @@ pub extern "C" fn assemble_modified_helmholtz_kernel_f32(
     let result =
         unsafe { ndarray::ArrayViewMut2::from_shape_ptr((ntargets, nsources), result_ptr) };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
-    make_modified_helmholtz_evaluator(sources, targets, omega).assemble_in_place(result, threading_type);
+    f32::assemble_kernel_in_place(sources, targets, result, KernelType::ModifiedHelmholtz(omega), num_threads);
 }
 
 /// Evaluate the modified Helmholtz potential sum (double precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `charge_ptr` - Pointer to a `(ncharge_vecs, nsources)` array of `ncharge_vecs` charge vectors.
@@ -521,7 +483,7 @@ pub extern "C" fn assemble_modified_helmholtz_kernel_f32(
 /// * `ntargets`   - Number of targets.
 /// * `ncharge_vecs` - Number of charge vectors.
 /// * `return_gradients` - If true return also the gradients.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn evaluate_modified_helmholtz_kernel_f64(
     source_ptr: *const f64,
@@ -533,20 +495,16 @@ pub extern "C" fn evaluate_modified_helmholtz_kernel_f64(
     ntargets: usize,
     ncharge_vecs: usize,
     return_gradients: bool,
-    parallel: bool,
+    num_threads: usize,
 ) {
-    use crate::kernels::EvalMode;
+
+    let kernel_type = KernelType::ModifiedHelmholtz(omega);
 
     let eval_mode = match return_gradients {
         true => EvalMode::ValueGrad,
         false => EvalMode::Value,
     };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
     let ncols: usize = match eval_mode {
         EvalMode::Value => 1,
         EvalMode::ValueGrad => 4,
@@ -560,19 +518,22 @@ pub extern "C" fn evaluate_modified_helmholtz_kernel_f64(
         ndarray::ArrayViewMut3::from_shape_ptr((ncharge_vecs, ntargets, ncols), result_ptr)
     };
 
-    make_modified_helmholtz_evaluator(sources, targets, omega).evaluate_in_place(
+    f64::evaluate_kernel_in_place(
+        sources,
+        targets,
         charges,
         result,
-        &eval_mode,
-        threading_type,
+        kernel_type,
+        eval_mode,
+        num_threads,
     );
 }
 
 /// Evaluate the modified Helmholtz potential sum (single precision version).
-/// 
-/// 
+///
+///
 /// # Arguments
-/// 
+///
 /// * `source_ptr` - Pointer to a `(3, nsources)` array of sources.
 /// * `target_ptr` - Pointer to a `(3, ntargets)` array of targets.
 /// * `charge_ptr` - Pointer to a `(ncharge_vecs, nsources)` array of `ncharge_vecs` charge vectors.
@@ -583,7 +544,7 @@ pub extern "C" fn evaluate_modified_helmholtz_kernel_f64(
 /// * `ntargets`   - Number of targets.
 /// * `ncharge_vecs` - Number of charge vectors.
 /// * `return_gradients` - If true return also the gradients.
-/// * `parallel`   - If true, assemble multithreaded, otherwise single threaded.
+/// * `num_threads`   - Number of threads to use
 #[no_mangle]
 pub extern "C" fn evaluate_modified_helmholtz_kernel_f32(
     source_ptr: *const f32,
@@ -595,20 +556,16 @@ pub extern "C" fn evaluate_modified_helmholtz_kernel_f32(
     ntargets: usize,
     ncharge_vecs: usize,
     return_gradients: bool,
-    parallel: bool,
+    num_threads: usize,
 ) {
-    use crate::kernels::EvalMode;
+
+    let kernel_type = KernelType::ModifiedHelmholtz(omega);
 
     let eval_mode = match return_gradients {
         true => EvalMode::ValueGrad,
         false => EvalMode::Value,
     };
 
-    let threading_type = match parallel {
-        true => ThreadingType::Parallel,
-        false => ThreadingType::Serial,
-    };
-
     let ncols: usize = match eval_mode {
         EvalMode::Value => 1,
         EvalMode::ValueGrad => 4,
@@ -622,10 +579,13 @@ pub extern "C" fn evaluate_modified_helmholtz_kernel_f32(
         ndarray::ArrayViewMut3::from_shape_ptr((ncharge_vecs, ntargets, ncols), result_ptr)
     };
 
-    make_modified_helmholtz_evaluator(sources, targets, omega).evaluate_in_place(
+    f32::evaluate_kernel_in_place(
+        sources,
+        targets,
         charges,
         result,
-        &eval_mode,
-        threading_type,
+        kernel_type,
+        eval_mode,
+        num_threads,
     );
 }
